==> bus-extra-8.0.5-sources.jar: org/miaixz/bus/extra/captcha/provider/PuzzleProvider.java <==
/*
 ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
 ~                                                                               ~
 ~ The MIT License (MIT)                                                         ~
 ~                                                                               ~
 ~ Copyright (c) 2015-2024 miaixz.org and other contributors.                    ~
 ~                                                                               ~
 ~ Permission is hereby granted, free of charge, to any person obtaining a copy  ~
 ~ of this software and associated documentation files (the "Software"), to deal ~
 ~ in the Software without restriction, including without limitation the rights  ~
 ~ to use, copy, modify, merge, publish, distribute, sublicense, and/or sell     ~
 ~ copies of the Software, and to permit persons to whom the Software is         ~
 ~ furnished to do so, subject to the following conditions:                      ~
 ~                                                                               ~
 ~ The above copyright notice and this permission notice shall be included in    ~
 ~ all copies or substantial portions of the Software.                           ~
 ~                                                                               ~
 ~ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR    ~
 ~ IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,      ~
 ~ FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE   ~
 ~ AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER        ~
 ~ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, ~
 ~ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN     ~
 ~ THE SOFTWARE.                                                                 ~
 ~                                                                               ~
 ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
 */
package org.miaixz.bus.extra.captcha.provider;

import org.miaixz.bus.core.lang.Symbol;
import org.miaixz.bus.extra.captcha.AbstractProvider;
import org.miaixz.bus.extra.captcha.strategy.CodeStrategy;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * 滑动验证码
 *
 * @author Kimi Liu
 * @since Java 17+
 */
public class PuzzleProvider extends AbstractProvider {

    public PuzzleProvider(int width, int height, int codeCount, int interfereCount) {
        super(width, height, codeCount, interfereCount);
    }

    public PuzzleProvider(int width, int height, CodeStrategy generator, int interfereCount) {
        super(width, height, generator, interfereCount);
    }

    @Override
    protected Image createImage(String code) {
        return null;
    }

    @Override
    public String get() {
        return null;
    }

    @Override
    public boolean verify(String inputCode) {
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i < inputCode.length(); i++) {
            char c = inputCode.charAt(i);
            if (c >= Symbol.C_ZERO && c <= Symbol.C_NINE) {
                list.add(Integer.valueOf(String.valueOf(c)));
            }
        }
        int sum = 0;
        for (Integer data : list) {
            sum += data;
        }
        double avg = sum * 1.0 / list.size();
        double sum2 = 0.0;
        for (Integer data : list) {
            sum2 += Math.pow(data - avg, 2);
        }
        double stddev = sum2 / list.size();
        return stddev != 0;
    }

}

==> bus-extra-8.3.3-sources.jar: org/miaixz/bus/extra/captcha/provider/PuzzleProvider.java <==
/*
 ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
 ~                                                                               ~
 ~ The MIT License (MIT)                                                         ~
 ~                                                                               ~
 ~ Copyright (c) 2015-2025 miaixz.org and other contributors.                    ~
 ~                                                                               ~
 ~ Permission is hereby granted, free of charge, to any person obtaining a copy  ~
 ~ of this software and associated documentation files (the "Software"), to deal ~
 ~ in the Software without restriction, including without limitation the rights  ~
 ~ to use, copy, modify, merge, publish, distribute, sublicense, and/or sell     ~
 ~ copies of the Software, and to permit persons to whom the Software is         ~
 ~ furnished to do so, subject to the following conditions:                      ~
 ~                                                                               ~
 ~ The above copyright notice and this permission notice shall be included in    ~
 ~ all copies or substantial portions of the Software.                           ~
 ~                                                                               ~
 ~ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR    ~
 ~ IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,      ~
 ~ FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE   ~
 ~ AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER        ~
 ~ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, ~
 ~ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN     ~
 ~ THE SOFTWARE.                                                                 ~
 ~                                                                               ~
 ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
*/
package org.miaixz.bus.extra.captcha.provider;

import java.awt.*;
import java.io.Serial;
import java.util.ArrayList;
import java.util.List;

import org.miaixz.bus.core.lang.Symbol;
import org.miaixz.bus.extra.captcha.AbstractProvider;
import org.miaixz.bus.extra.captcha.strategy.CodeStrategy;

/**
 * Sliding CAPTCHA Provider.
 *
 * @author Kimi Liu
 * @since Java 17+
 */
public class PuzzleProvider extends AbstractProvider {

    @Serial
    private static final long serialVersionUID = -2852291851366L;

    /**
     * Constructor.
     *
     * @param width          Image width.
     * @param height         Image height.
     * @param codeCount      Number of characters.
     * @param interfereCount Number of interfering elements.
     */
    public PuzzleProvider(int width, int height, int codeCount, int interfereCount) {
        super(width, height, codeCount, interfereCount);
    }

    /**
     * Constructor.
     *
     * @param width          Image width.
     * @param height         Image height.
     * @param generator      CAPTCHA code generator.
     * @param interfereCount Number of interfering elements.
     */
    public PuzzleProvider(int width, int height, CodeStrategy generator, int interfereCount) {
        super(width, height, generator, interfereCount);
    }

    /**
     * Creates the CAPTCHA image.
     *
     * @param code The CAPTCHA code to be rendered in the image.
     * @return The generated CAPTCHA image.
     */
    @Override
    protected Image createImage(String code) {
        // TODO: Implementation for creating the puzzle CAPTCHA image goes here.
        return null;
    }

    /**
     * Gets the CAPTCHA code or related data.
     *
     * @return A string representing the CAPTCHA data to be verified.
     */
    @Override
    public String get() {
        // TODO: Implementation for getting the CAPTCHA code/data goes here.
        return null;
    }

    /**
     * Verifies the user's input against the generated CAPTCHA.
     *
     * @param inputCode The code provided by the user.
     * @return {@code true} if the verification is successful, {@code false} otherwise.
     */
    @Override
    public boolean verify(String inputCode) {
        // This is a placeholder verification logic.
        // It parses digits from the input string, calculates the standard deviation,
        // and returns true if the standard deviation is not zero.
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i < inputCode.length(); i++) {
            char c = inputCode.charAt(i);
            if (c >= Symbol.C_ZERO && c <= Symbol.C_NINE) {
                list.add(Integer.valueOf(String.valueOf(c)));
            }
        }

        if (list.isEmpty()) {
            return false;
        }

        int sum = 0;
        for (Integer data : list) {
            sum += data;
        }
        double avg = sum * 1.0 / list.size();
        double sum2 = 0.0;
        for (Integer data : list) {
            sum2 += Math.pow(data - avg, 2);
        }
        double stddev = sum2 / list.size();
        return stddev != 0;
    }

}
